/*
 sums.c
*/

#include "qpapi.h"
#include "mathl.h"
#include "strings.h"
#include "error.h"    /* error message strings for toolkit */

/*
Here are the names for the addin functions.
The names must be in upper case.

Each name is followed by an argument type descriptor word.
The argument type uses a macro defined and explained in qpapi.h.
*/

const struct names {
   char name[MaxNameLen];
   int type;
}

funcs[] = {
{       "AUTHOR", ArgType(0,0,0,0,0,0,0) },
{       "LIST", ArgType(0,0,0,0,0,0,0) },
{       "SUMDIFF", ArgType(2,argBlock,argBlock,0,0,0,0) },
{       "SUMQUOT", ArgType(2,argBlock,argBlock,0,0,0,0) },
{       "SUMSQUARE", ArgType(1,argBlock,0,0,0,0,0) },

};

// The following functions (Init, Term, Names, Eval) are required.

int addin::Init()
// ret # of functions
{
   return DIM(funcs);
}

void addin::Term()
{
   /* release memory */
   resize(0);
   return;
}

int addin::Names(int k, struct functable *p)
// called with k < number returned by Init
{
   p->entry = 0;
   p->purity = PureFunction;
   p->argtype = funcs[k].type;
   strncpy((char *)p->name.s,funcs[k].name,MaxNameLen);
   p->name.len = strlen(funcs[k].name);
   return 0;
}

long addin::Eval()
{
/******

   a_name is the name of the library author
   f_names is a listing of the functions in the QLL.
 
   Be sure that you allocate only enough space to cover the initilizers
   for these two strings.  It stops rude things from happening.

******/

char a_name[11] = "R. Wilkman\0";
char f_names[40] = "AUTHOR LIST SUMDIFF SUMQUOT SUMSQUARE \0";

/* SUMDIFF variables */
/* function for calculating the sum of the differences */
block sdblk1, sdblk2;
cell curr1,curr2;
int row1,row2,i,j;
float sdtmp1,sdtmp2,sdtot;

/* SUMQUOT variables */
/* function for calculating the sum of the quotients */
block sqblk1, sqblk2;
cell qcurr1,qcurr2;
int qrow1,qrow2;
float sqtmp1,sqtmp2,sqtot;

/* SUMSQUARE variables */
/* function for calculating the sum of the squares */
block ssblk;
cell sscurr;
int ssrow, sscol;
float sstmp, sstot;

   switch (function)
   {
   case 0:  /* AUTHOR */
   ctop(a_name);
   return retString(a_name);
   
   case 1:  /* LIST */
   ctop(f_names);
   return retString(f_names);

   case 2: /* SUMDIFF */
   sdblk1 = * getArgBlock(0);
   sdblk2 = * getArgBlock(1);
   row1 = sdblk1.rows();
   row2 = sdblk2.rows();

   if(row1 != row2){
   ctop(MSG2);
   return retString(MSG2);
   }

   for(i=0;i<row1;i++){
      curr1 = sdblk1.getCell(i,0);
      curr2 = sdblk2.getCell(i,0);
      sdtmp1 = getCellReal(curr1);
      sdtmp2 = getCellReal(curr2);
      sdtot += (sdtmp1-sdtmp2);
   }
   return retReal(sdtot);

   case 3: /* SUMQUOT */
   sqblk1 = * getArgBlock(0);
   sqblk2 = * getArgBlock(1);
   qrow1 = sqblk1.rows();
   qrow2 = sqblk2.rows();
 
   if(qrow1 != qrow2){
   ctop(MSG2);
   return retString(MSG2);
   }
 
   for(i=0;i<qrow1;i++){
      qcurr1 = sqblk1.getCell(i,0);
      qcurr2 = sqblk2.getCell(i,0);
      sqtmp1 = getCellReal(qcurr1);
      sqtmp2 = getCellReal(qcurr2);

      if(sqtmp2 == 0){
         ctop(MSG3);
         return retString(MSG3);
      }

      sqtot += (sqtmp1/sqtmp2);
   }
   return retReal(sqtot);
 
   case 4:  /* SUMSQUARE */
   ssblk = * getArgBlock(0);
   ssrow = ssblk.rows();
   sscol = ssblk.columns();
   sstmp = 0; sstot = 0;

   for(i=0;i<sscol;++i)
      for(j=0;j<ssrow;++j){
         sscurr = ssblk.getCell(j,i);
         sstmp = getCellReal(sscurr);
         sstot += (sstmp*sstmp);
         }
   return retReal(sstot);


   } // end of switch
   // not reached
   return retERR();
}
